package nl.dancingbear.tunit.example;
import nl.dancingbear.tunit.StackTraceUtil;
import nl.dancingbear.tunit.TUnitTestCase;
import nl.dancingbear.tunit.ToolStub;
import aterm.ATerm;

/**
 * An example test case to demonstrate the toolbus testing framework.
 *
 * @author Bas Basten & Anton Lycklama a Nijeholt
 * @date 23-02-2007
 */
public class ExampleTestCase extends TUnitTestCase {
    /**
     * The function that contains the actual test case.
     *
     * @author Bas Basten
     * @author Anton Lycklama a Nijeholt
     * @date 21-02-2007
     */
    public void testSendValue() {
        try {
            ToolStub tool1 = this.createToolStub("tool1");
            ToolStub tool2 = this.createToolStub("tool2");

            this.connectToolStubs();

            ATerm sendValue = this.factory.make("sendValue(<int>)", new Integer(10));
            ATerm showValue = this.factory.make("showValue(<int>)", new Integer(10));

            // Here we simulate the behaviour of the tools that would normally
            // communicate with the toolbus processes that we would like to test.
            // We replay all the events, evals, do's etc. that would normally be
            // communicated between the to processes and their tools.

            // Scenario 1: send a value to proces 1 and see if it is properly
            // relayed to the tool of process 2.
            tool1.sendEvent(sendValue);
            tool1.expectAckEvent(sendValue, 3000);
            tool2.expectEval(showValue, 3000);
            tool2.sendValue(showValue);

            // Scenario 2: send a value to proces 2 and see if it is properly
            // relayed to the tool of process 1.
            tool2.sendEvent(sendValue);
            tool2.expectAckEvent(sendValue, 3000);
            tool1.expectDo(showValue, 3000);

            // If we have received all the expected ack-events, evals and do's
            // before the timeouts have exceeded, our test has passed.
            this.disconnectToolStubs();

            System.out.println("Test Passed");
        }
        catch (Exception ex) {
            System.out.println(StackTraceUtil.getStackTrace(ex));
            fail(ex.toString());
        }
    }

    /**
     * Let the JUnit framework start the toolbus at a given port and load the
     * initialization test script. It is important to stop the toolbus if an
     * exception arises! If the exception is not caught the toolbus process will
     * still be running. If we re-run the toolbus process the toolbus will be
     * initialized at a new port because there is a toolbus process still running
     * at the given port.
     *
     * @author Bas Basten
     * @author Anton Lycklama a Nijeholt
     * @date 23-02-2007
     */
    protected void setUp() {
        // Get the absolute path of the top directory of the current package.
        // For example: /tmp/<user>/metarepos/tunit
        String topSrcDir = getTopSrcDir();

        System.out.println(topSrcDir);

        try {
            this.startToolbus(topSrcDir + "/src/nl/dancingbear/tunit/example/", topSrcDir + "/src/nl/dancingbear/tunit/example/init.tb", 7000);
        }
        catch (Exception ex) {
            System.out.println(StackTraceUtil.getStackTrace(ex));
            this.stopToolbus();
            fail(ex.toString());
        }
    }

    /**
     * Clean up the allocated resources at the end of the test.
     *
     * @author Bas Basten
     * @author Anton Lycklama a Nijeholt
     * @date 23-02-2007
     */
    protected void tearDown() {
        this.stopToolbus();
    }
}
